from rest_framework import status
import logging

logger = logging.getLogger(__name__) 

class GetRolePermissionsHelper:        
    def handle_get_service_permissions(role):
        from permissions_mngmt.models import RolesServicePermissions
        role_service_perms = RolesServicePermissions.get_role_service_permission_by_role(role)
        if role_service_perms:
            return role_service_perms
        else:
            print (f"No service permissions for this role found")
            logger.error(f"No service permissions for this role found")
            return None
        
    def handle_get_model_permissions(role):
        from permissions_mngmt.models import RolesModelPermissions 
        role_model_perms = RolesModelPermissions.get_role_model_permission_by_role(role)
        if role_model_perms:
            return role_model_perms
        else:
            print (f"No model permissions for this role found")
            logger.error(f"No model permissions for this role found")
            return None

# Added User to Role
class AddAppUsersPermissionsHelper:
    def handle_add_service_permissions(role, app_user):
        from permissions_mngmt.serializers import AppUserServicePermissionsCreateSerializer
        # return True
        role_service_perms = GetRolePermissionsHelper.handle_get_service_permissions(role)
        perms_to_add = []
        if role_service_perms:
            for perm in role_service_perms:
                perms_to_add.append({                    
                    'app_user': app_user,
                    'role': role,
                    'service_permission': perm.service_permission_id
                })
            if perms_to_add:
                serialized_perms = AppUserServicePermissionsCreateSerializer(data=perms_to_add, many=True)
                if serialized_perms.is_valid():
                    serialized_perms.save()
                    return True
                else:
                    print (f"\n\nApp Users Service Permissions Serializer Error: {serialized_perms.errors}")
                    logger.error(f"App Users Service Permissions Serializer Serializer Error: {serialized_perms.errors}")
                    return False
            else:
                print (f"No permissions to add")
                logger.error(f"No permissions to add")
                return False
        else:
            print (f"No service permissions for this role found")
            logger.error(f"No service permissions for this role found")
            return False
    
    def handle_add_model_permissions(role, app_user):
        from permissions_mngmt.serializers import AppUsersModelPermissionsCreateSerializer
        # return True
        role_model_perms = GetRolePermissionsHelper.handle_get_model_permissions(role)
        model_perms_to_add = []
        if role_model_perms:
            for perm in role_model_perms:
                model_perms_to_add.append({
                    'model_permission': perm.model_permission.model_permission_id,
                    'app_user': app_user,
                    'role': role
                })               
            if model_perms_to_add:
                serialized_model_perms = AppUsersModelPermissionsCreateSerializer(data=model_perms_to_add, many=True)
                if serialized_model_perms.is_valid():
                    serialized_model_perms.save()
                    return True
                else:
                    print (f"\n\nApp Users Model Permissions Serializer Error: {serialized_model_perms.errors}")
                    logger.error(f"App Users Model Permissions Serializer Error: {serialized_model_perms.errors}")
                    return False
            else:
                print (f"No model permissions to add")
                logger.error(f"No model permissions to add")
                return False
        else:
            print (f"No model permissions for this role found")
            logger.error(f"No model permissions for this role found")
            return False

# Remove User from Role
class RemoveAppUserPermissionsHelper:
    def handle_remove_app_user_service_permissions(old_role, app_user):
        from permissions_mngmt.models import RolesServicePermissions, AppUserServicePermissions
        # return True
        # Get List of Role Service Permissions
        service_perms = RolesServicePermissions.get_role_service_permission_by_role(old_role.role_id)
        if not service_perms:
            print (f"No service permissions for this role found")
            logger.error(f"No service permissions for this role found")
        # Get List of User Service Permissions
        app_user_service_perms = AppUserServicePermissions.get_app_user_service_permission_by_app_user(app_user)
        if not app_user_service_perms:
            print (f"No service permissions for this user found")
            logger.error(f"No service permissions for this user found")
        # Remove User Service Permissions that match Role Service Permissions  
        for app_user_perm in app_user_service_perms:
            for role_perm in service_perms:
                if app_user_perm.service == role_perm.service:
                    app_user_perm.delete_employee_service_permission()
        check_app_user_service_perms = AppUserServicePermissions.get_app_user_service_permission_by_app_user(app_user)
        if len(check_app_user_service_perms) == 0:
            return True  
        elif len(check_app_user_service_perms) > 0:
            return False    

    def handle_remove_app_user_model_permissions(old_role, app_user):
        from permissions_mngmt.models import RolesModelPermissions, AppUsersModelPermissions
        from auth_n_auth.models import AppUserRoles, Roles
        # return True
        role = None
        model_perms = None

        # Get Role for User
        app_user_role = AppUserRoles.get_user_roles_by_app_user(app_user).filter(role=old_role).first()
        if app_user_role:
            print (f"Role for User: {app_user_role}")            
            role_for_app_user = app_user_role.role
            role_id = role_for_app_user.role_id
            role = Roles.get_role_by_id(role_id)
            if role:
                # Get List of Role Model Permissions
                model_perms = RolesModelPermissions.get_role_model_permission_by_role(role)
                if model_perms:  
                    # Get List of App User Model Permissions
                    app_user_model_perms = AppUsersModelPermissions.get_employee_model_permission_by_employee(app_user)                    
                    if app_user_model_perms:
                        # Remove Users Model Permissions that match Role Model Permissions  
                        for app_user_perm in app_user_model_perms:
                            for role_perm in model_perms:
                                if app_user_perm.model.model_permission_id == role_perm.model_permission_id:
                                    app_user_perm.delete_app_user_model_permission()
                        check_app_user_model_perms = AppUsersModelPermissions.get_app_users_model_permission_by_app_user(app_user)
                        if len(check_app_user_model_perms) == 0:
                            return True
                        elif len(check_app_user_model_perms) > 0:
                            return False

# Deleted Role
class RemoveRolePermissionsHelper:
    # Get List of Users with Role
    def get_app_users_list_by_role(role):
        from auth_n_auth.models import AppUserRoles
        app_users_roles = AppUserRoles.get_user_role_by_role_id(role)
        app_users_list = []
        if app_users_roles:
            for app_user in app_users_roles:
                print (f"User: {app_user.id}")
                app_users_list.append(app_user.id)
            if app_users_list:
                return app_users_list
            else:
                print (f"No users with this role found")
                logger.error(f"No users with this role found")
                return None
        else:
            print (f"No users with this role found")
            logger.error(f"No users with this role found")
            return None

    # Remove Service Permissions for Employees with Role
    def handle_remove_service_permissions(role):        
        app_users_in_role = RemoveRolePermissionsHelper.get_app_users_list_by_role(role)
        
        for app_user in app_users_in_role:
            RemoveAppUserPermissionsHelper.handle_remove_app_user_service_permissions(app_user)

    # Remove Model Permissions for Employees with Role
    def handle_remove_model_permissions(role):        
        app_users_in_role = RemoveRolePermissionsHelper.get_app_users_list_by_role(role)
        
        for app_user in app_users_in_role:
            RemoveAppUserPermissionsHelper.handle_remove_app_user_model_permissions(app_user)
    
    # Remove Service Permissions for Role
    def handle_remove_service_permssions_for_role(role):
        role_service_perms = GetRolePermissionsHelper.handle_get_service_permissions(role)
        if role_service_perms:
            for perm in role_service_perms:
                perm.delete_role_service_permission()
        else:
            print (f"No service permissions for this role found")
            logger.error(f"No service permissions for this role found")
            return None
        
    # Remove Model Permissions for Role    
    def handle_remove_model_permssions_for_role(role):
        role_model_perms = GetRolePermissionsHelper.handle_get_model_permissions(role)
        if role_model_perms:
            for perm in role_model_perms:
                perm.delete_role_model_permission()
        else:
            print (f"No model permissions for this role found")
            logger.error(f"No model permissions for this role found")
            return None
        
    def handle_remove_permissions_for_deleted_role(role):
        # Remove Permissions for the Employees with the Deleted Role
        RemoveRolePermissionsHelper.handle_remove_service_permissions(role)
        RemoveRolePermissionsHelper.handle_remove_model_permissions(role)

        # Remove the RoleServicePerms for the Deleted Role
        RemoveRolePermissionsHelper.handle_remove_service_permssions_for_role(role)

        # Remove the RoleModelPerms for the Deleted Role
        RemoveRolePermissionsHelper.handle_remove_model_permssions_for_role(role)

class CheckPermissionsHelper:
    @staticmethod
    def verify_permissions(permission_name, crud_permission, permission=None):
        """
        Verifies if the user has the required permissions.

        Args:
        - user: The user object.
        - permission: The permission object.
        - permission_name: The name of the permission.
        - crud_permission: The CRUD permissions required.
        - special_permission: Optional special permission object.

        Returns:
        - True if the user has the required permissions, False otherwise.
        """        
        name_match = permission.get('name') == permission_name
        crud_match = permission.get('crud_permissions') == crud_permission
        if not name_match:
            return False    
        if not crud_match:
            return False
        if name_match and crud_match:
            return True    
        return None  
 
class GetUserPermissionsHelper:
    @staticmethod
    def get_user_service_permissions(user_id, user_roles):
        from permissions_mngmt.serializers import ServicesPermissionsReadSerializer
        from permissions_mngmt.models import AppUserServicePermissions
        """
        Gets the permissions for the user.

        Args:
        - user: The user object.

        Returns:
        - The permissions for the user.
        """
        service_perms = []
        msg = ""
        status_code = status.HTTP_100_CONTINUE
        if not user_roles:
            print (f"\nNo roles found for user")
            logger.error(f"No roles found for user")
            msg = "No roles found for user"
            status_code = status.HTTP_404_NOT_FOUND
            return msg, status_code, False
        if not user_id:
            print (f"\nNo user id found")
            logger.error(f"No user id found")
            msg = "No user id found"
            status_code = status.HTTP_404_NOT_FOUND
            return msg, status_code, False
        for role_id in user_roles:
            user_service_perms = AppUserServicePermissions.get_app_user_service_permission_by_role_app_user(role_id, user_id)
            if user_service_perms:
                for perm in user_service_perms:
                    if perm.service_permission.service_permission not in service_perms:
                        service_perms.append(perm.service_permission.service_permission)
            else:
                print (f"\nNo service permissions found for role")
                logger.error(f"No service permissions found for role")
                msg = "No service permissions found for role"
                status_code = status.HTTP_404_NOT_FOUND
                return msg, status_code, False
        if not service_perms:
            print (f"\nNo service permissions found for user")
            logger.error(f"No service permissions found for user")
            msg = "No service permissions found for user"
            status_code = status.HTTP_404_NOT_FOUND
            return msg, status_code, False
        else:
            try:
                serialized_service_permissions = ServicesPermissionsReadSerializer(service_perms, many=True)
                return serialized_service_permissions, status.HTTP_200_OK, True
            except Exception as e:
                print (f"\nError serializing service permissions: {e}")
                logger.error(f"Error serializing service permissions: {e}")
                msg = "Error serializing service permissions"
                status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
                return msg, status_code, False

    @staticmethod
    def get_user_model_permissions(user_id, user_roles):
        """
        Gets the permissions for the user.

        Args:
        - user: The employee object.

        Returns:
        - The permissions for the user.
        """
        from permissions_mngmt.serializers import ModelsPermissionsReadSerializer
        from permissions_mngmt.models import AppUsersModelPermissions
        model_perms = []
        msg = ""
        status_code = status.HTTP_100_CONTINUE
        if not user_roles:
            print (f"No roles found for user")
            logger.error(f"No roles found for user")
            msg = "No roles found for user"
            status_code = status.HTTP_404_NOT_FOUND
            return msg, status_code, False
        if not user_id:
            print (f"No user id found")
            logger.error(f"No user id found")
            msg = "No user id found"
            status_code = status.HTTP_404_NOT_FOUND
            return msg, status_code, False
        for role_id in user_roles:
            user_model_perms = AppUsersModelPermissions.get_app_users_model_permission_by_role_app_user(role_id, user_id)
            if user_model_perms:
                for perm in user_model_perms:
                    if perm.model_permission.model_permission not in model_perms:
                        model_perms.append(perm.model_permission.model_permission)
            else:
                print (f"No model permissions found for role")
                logger.error(f"No model permissions found for role")
                msg = "No model permissions found for role"
                status_code = status.HTTP_404_NOT_FOUND
                return msg, status_code, False
        if not model_perms:
            print (f"No model permissions found for user")
            logger.error(f"No model permissions found for user")
            msg = "No model permissions found for user"
            status_code = status.HTTP_404_NOT_FOUND
            return msg, status_code, False
        else:
            try:
                serialized_model_permissions = ModelsPermissionsReadSerializer(model_perms, many=True)
                return serialized_model_permissions, status.HTTP_200_OK, True
            except Exception as e:
                print (f"Error serializing model permissions: {e}")
                logger.error(f"Error serializing model permissions: {e}")
                msg = "Error serializing model permissions"
                status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
                return msg, status_code, False
        
